/*
 * $Id: vlan.h,v 1.8 Broadcom SDK $
 *
 * $Copyright: Copyright 2015 Broadcom Corporation.
 * This program is the proprietary software of Broadcom Corporation
 * and/or its licensors, and may only be used, duplicated, modified
 * or distributed pursuant to the terms and conditions of a separate,
 * written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized
 * License, Broadcom grants no license (express or implied), right
 * to use, or waiver of any kind with respect to the Software, and
 * Broadcom expressly reserves all rights in and to the Software
 * and all intellectual property rights therein.  IF YOU HAVE
 * NO AUTHORIZED LICENSE, THEN YOU HAVE NO RIGHT TO USE THIS SOFTWARE
 * IN ANY WAY, AND SHOULD IMMEDIATELY NOTIFY BROADCOM AND DISCONTINUE
 * ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization,
 * constitutes the valuable trade secrets of Broadcom, and you shall use
 * all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of
 * Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS
 * PROVIDED "AS IS" AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES,
 * REPRESENTATIONS OR WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY,
 * OR OTHERWISE, WITH RESPECT TO THE SOFTWARE.  BROADCOM SPECIFICALLY
 * DISCLAIMS ANY AND ALL IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
 * NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES,
 * ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION OR
 * CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING
 * OUT OF USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL
 * BROADCOM OR ITS LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL,
 * INCIDENTAL, SPECIAL, INDIRECT, OR EXEMPLARY DAMAGES WHATSOEVER
 * ARISING OUT OF OR IN ANY WAY RELATING TO YOUR USE OF OR INABILITY
 * TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF
 * THE AMOUNT ACTUALLY PAID FOR THE SOFTWARE ITSELF OR USD 1.00,
 * WHICHEVER IS GREATER. THESE LIMITATIONS SHALL APPLY NOTWITHSTANDING
 * ANY FAILURE OF ESSENTIAL PURPOSE OF ANY LIMITED REMEDY.$
 *
 */

#ifndef _BRDIMPL_VLAN_H_
#define _BRDIMPL_VLAN_H_

#ifdef CFG_SWITCH_VLAN_INCLUDED
/*
 * SW VLAN shadow table :
 *      - Used to serve the Port-based and 1Q-based VLAN
 *
 * Note :
 *  1. This shadow table will be at reset status when VLAN type changed.
 *  2. Member ports is represented in logical port list format.
 */
typedef struct vlan_list_s {
    uint16  vlan_id;    /* be the VLAN group index in PVLAN and VID in QVLAN */
    uint8   uplist[MAX_UPLIST_WIDTH]; /* the member port list in this VLAN group */
    struct vlan_list_s *next;
} vlan_list_t;

/* Integrated VLAN SW information :
 *  - All related VLAN configuration or informaiton for both Port-based
 *      and 1Q-based VLAN.
 */
typedef struct vlan_info_s {
    vlan_type_t type;   /* the current working VLAN type */
    uint16      count;  /* the number of the created VLAN groups */
    uint16      max_count;  /* the allowed MAX VLAN groups */
    vlan_list_t *head; /* head of vlan list */
    vlan_list_t *tail; /* tail of vlan list */
} vlan_info_t;

#define BOARD_VLAN_GROUP_EXIST_CHK(_vlan_list)      ((_vlan_list) != NULL)

#define MAX_QVLAN_ID    4094

/*
 * VLAN reset
 */
extern sys_error_t brdimpl_vlan_reset(void) REENTRANT;

/*
 * VLAN type
 */
extern sys_error_t brdimpl_vlan_type_set(vlan_type_t type) REENTRANT;
extern sys_error_t brdimpl_vlan_type_get(vlan_type_t *type) REENTRANT;

/*
 * VLAN creation/deletion
 */
extern sys_error_t brdimpl_vlan_create(uint16 vlan_id) REENTRANT;
extern sys_error_t brdimpl_vlan_destroy(uint16 vlan_id) REENTRANT;

/*
 * Port-base VLAN port set/get
 */
extern sys_error_t brdimpl_pvlan_port_set(uint16  vlan_id,
                                          uint8 *uplist) REENTRANT;
extern sys_error_t brdimpl_pvlan_port_get(uint16  vlan_id,
                                          uint8 *uplist) REENTRANT;

                                          /*
 * Port-base VLAN egress ports get
 */
extern sys_error_t brdimpl_pvlan_egress_get(uint16 uport,
                                            uint8 *uplist) REENTRANT;

/*
 * 8021Q VLAN port set/get
 */
extern sys_error_t brdimpl_qvlan_port_set(uint16  vlan_id, uint8 *uplist,
                                           uint8 *taguplist) REENTRANT;
extern sys_error_t brdimpl_qvlan_port_get(uint16  vlan_id, uint8 *uplist,
                                           uint8 *taguplist) REENTRANT;

/*
 * retrieve total vlan numbers
 */
extern uint16 brdimpl_vlan_count(void) REENTRANT;


/*
 * retrieve vlan members with index
 */
extern sys_error_t brdimpl_pvlan_get_by_index(uint16  index, uint16 *vlan_id,
                                           uint8 *uplist) REENTRANT;
extern sys_error_t brdimpl_qvlan_get_by_index(uint16  index, uint16 *vlan_id,
                                     uint8 *uplist, uint8 *taguplist) REENTRANT;

/* _brdimpl_vlan_init() is used for system bootup process only. */
extern sys_error_t _brdimpl_vlan_init(void) REENTRANT;
#ifdef BRD_VLAN_DEBUG
/* _brdimpl_dump_vlan_info() is used for debug purpose. */
extern void _brdimpl_dump_vlan_info(void) REENTRANT;
#endif  /* BRD_VLAN_DEBUG */

#endif  /* CFG_SWITCH_VLAN_INCLUDED */

#endif /* _BRDIMPL_VLAN_H_ */
