/*
 * $Id: uip_timers.h,v 1.8 Broadcom SDK $
 *
 * $Copyright: Copyright 2015 Broadcom Corporation.
 * This program is the proprietary software of Broadcom Corporation
 * and/or its licensors, and may only be used, duplicated, modified
 * or distributed pursuant to the terms and conditions of a separate,
 * written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized
 * License, Broadcom grants no license (express or implied), right
 * to use, or waiver of any kind with respect to the Software, and
 * Broadcom expressly reserves all rights in and to the Software
 * and all intellectual property rights therein.  IF YOU HAVE
 * NO AUTHORIZED LICENSE, THEN YOU HAVE NO RIGHT TO USE THIS SOFTWARE
 * IN ANY WAY, AND SHOULD IMMEDIATELY NOTIFY BROADCOM AND DISCONTINUE
 * ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization,
 * constitutes the valuable trade secrets of Broadcom, and you shall use
 * all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of
 * Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS
 * PROVIDED "AS IS" AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES,
 * REPRESENTATIONS OR WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY,
 * OR OTHERWISE, WITH RESPECT TO THE SOFTWARE.  BROADCOM SPECIFICALLY
 * DISCLAIMS ANY AND ALL IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
 * NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES,
 * ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION OR
 * CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING
 * OUT OF USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL
 * BROADCOM OR ITS LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL,
 * INCIDENTAL, SPECIAL, INDIRECT, OR EXEMPLARY DAMAGES WHATSOEVER
 * ARISING OUT OF OR IN ANY WAY RELATING TO YOUR USE OF OR INABILITY
 * TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF
 * THE AMOUNT ACTUALLY PAID FOR THE SOFTWARE ITSELF OR USD 1.00,
 * WHICHEVER IS GREATER. THESE LIMITATIONS SHALL APPLY NOTWITHSTANDING
 * ANY FAILURE OF ESSENTIAL PURPOSE OF ANY LIMITED REMEDY.$
 *
 */

#ifndef _UIP_TIMERS_H_
#define _UIP_TIMERS_H_

#include "system.h"

/*
 * Units of current system time/ticks
 */
typedef tick_t clock_time_t;

/*
 * Ticks per second
 */
#define CLOCK_SECOND        SAL_USEC_TO_TICKS(1000000UL)
#define clock_time()        sal_get_ticks()

/*
 * Timer in ticks
 */
struct timer {
    clock_time_t start;
    clock_time_t interval;
};
#define timer_set(ptimer, interv)       do {                        \
    (ptimer)->start = sal_get_ticks();                              \
    (ptimer)->interval = (interv);                                  \
} while(0)
#define timer_reset(ptimer)             do {                        \
    (ptimer)->start = sal_get_ticks();                              \
} while(0)
#define timer_expired(ptimer)                                       \
    SAL_TIME_EXPIRED((ptimer)->start, (ptimer)->interval)
#define timer_remaining(ptimer)                                     \
    ((ptimer)->start + (ptimer)->interval - sal_get_ticks())
    
/*
 * Timer in seconds
 */
struct stimer {
    clock_time_t start;
    clock_time_t interval;
};
#define stimer_set(ptimer, interv)      do {                        \
    (ptimer)->start = sal_get_ticks();                              \
    (ptimer)->interval = (interv) * CLOCK_SECOND;                   \
} while(0)
#define stimer_reset(ptimer)            do {                        \
    (ptimer)->start = sal_get_ticks();                              \
} while(0)
#define stimer_expired(ptimer)                                      \
    SAL_TIME_EXPIRED((ptimer)->start, (ptimer)->interval)
#define stimer_remaining(ptimer)                                    \
    (((ptimer)->start + (ptimer)->interval - sal_get_ticks() - 1)   \
        / CLOCK_SECOND + 1)

/*
 * Event timers (some hard mappings required) 
 */
#define etimer                          timer
#define etimer_set                      timer_set
#define etimer_reset                    timer_reset
#define etimer_expired(ptimer)                                      \
    ((ptimer)->interval != 0 &&                                     \
     SAL_TIME_EXPIRED((ptimer)->start, (ptimer)->interval))
#define etimer_stop(ptimer)             do  {                       \
    (ptimer)->interval = 0;                                         \
} while(0)

#endif /* _UIP_TIMERS_H_ */
